"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.showHelp = exports.command = exports.argv = exports.commands = void 0;
const yargs_1 = __importDefault(require("yargs/yargs"));
const helpers_1 = require("yargs/helpers");
exports.commands = ['dump', 'restore', 'sync'];
const _yargs = (0, yargs_1.default)((0, helpers_1.hideBin)(process.argv))
    .command('dump', 'Dump data from redis index to dump file, using custom pattern.', {
    f: {
        alias: 'filePath',
        description: 'Dump file full path, for example `~/dump.csv`.',
        demandOption: true,
    },
    u: {
        alias: 'uri',
        description: 'Redis uri with db index, for example `redis://localhost:6379/0`.',
        demandOption: true,
    },
    p: {
        alias: 'pattern',
        default: '*',
        description: 'Redis key pattern, for example `username:*` and `*` means all keys.',
    },
    b: {
        alias: 'bulkSize',
        default: 1000,
        description: 'Count of item to get in parallel.',
    },
})
    .command('restore', 'Restore from csv dump file.', {
    f: {
        alias: 'filePath',
        description: 'Dump file full path, for example `~/dump.csv`.',
        demandOption: true,
    },
    u: {
        alias: 'uri',
        description: 'Redis uri with db index, for example `redis://localhost:6379/0`.',
        demandOption: true,
    },
    t: {
        alias: 'useTtl',
        default: true,
        description: 'Use Redis ttl instead of expiration time.',
    },
    b: {
        alias: 'bulkSize',
        default: 1000,
        description: 'Count of item to insert in parallel.',
    },
})
    .command('sync', 'Sync between two redis, using custom pattern.', {
    su: {
        alias: 'sourceUri',
        description: 'Source Redis uri with db index, for example `redis://localhost:6379/0`.',
        demandOption: true,
    },
    tu: {
        alias: 'targetUri',
        description: 'Target Redis uri with db index, for example `redis://localhost:6380/0`.',
        demandOption: true,
    },
    p: {
        alias: 'pattern',
        default: '*',
        description: 'Redis key pattern, for example `username:*` and `*` means all keys.',
    },
    b: {
        alias: 'bulkSize',
        default: 1000,
        description: 'Count of item to sync in parallel.',
    },
    t: {
        alias: 'useTtl',
        default: true,
        description: 'Use Redis ttl instead of expiration time.',
    },
})
    .demandCommand(1, 'You have to pass one command with related arguments!')
    .check((argv, options) => {
    const _commands = argv._;
    const command = _commands === null || _commands === void 0 ? void 0 : _commands.at(0);
    if (_commands.length > 1) {
        throw new Error(`Only one command may be passed: ${_commands.join(', ')}.`);
    }
    else if (!exports.commands.includes(command)) {
        throw new Error(`Unknown command: ${command}`);
    }
    else {
        return true;
    }
})
    .usage('$0 <command> <arguments>')
    .help();
exports.argv = _yargs.parseSync();
exports.command = (_a = exports.argv === null || exports.argv === void 0 ? void 0 : exports.argv._) === null || _a === void 0 ? void 0 : _a.at(0);
exports.showHelp = _yargs.showHelp;
//# sourceMappingURL=argv.js.map