"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.countLines = exports.readStream = exports.writeStream = void 0;
const path_1 = require("path");
const fs_1 = require("fs");
const event_stream_1 = require("event-stream");
const events_1 = __importDefault(require("events"));
const logeError = (func, data, reason) => {
    console.error(`[RedisDRS] Error while run ${func} callback: data > ${data}`);
    console.error(`[RedisDRS] Error while run ${func} callback: message > ${reason}`);
};
function writeStream(filePath) {
    const stream = (0, fs_1.createWriteStream)((0, path_1.resolve)(filePath));
    return {
        write: (data) => __awaiter(this, void 0, void 0, function* () {
            data = yield data;
            if (typeof data === 'object')
                data = JSON.stringify(data);
            if (!stream.write(data + '\r\n')) {
                return new Promise((res, rej) => {
                    stream.once('drain', () => res(data));
                });
            }
            else {
                return Promise.resolve(data);
            }
        }),
        end: () => stream.end(),
    };
}
exports.writeStream = writeStream;
function readStream({ filePath, bulkSize = 1000, callback, }) {
    const emitter = new events_1.default();
    const reader = () => {
        return new Promise((res, rej) => __awaiter(this, void 0, void 0, function* () {
            const _filePath = (0, path_1.resolve)(filePath);
            let linesCount = yield (0, exports.countLines)(_filePath);
            emitter.emit('linesCount', linesCount);
            let index = 0;
            let pool = [];
            const stream = (0, fs_1.createReadStream)(_filePath)
                .pipe((0, event_stream_1.split)())
                .pipe((0, event_stream_1.mapSync)((line) => __awaiter(this, void 0, void 0, function* () {
                const tempIndex = ++index;
                line = line === null || line === void 0 ? void 0 : line.trim();
                // Skip empty lines
                if (!line)
                    return;
                emitter.emit('line', index, line);
                pool.push(callback(tempIndex, line).catch((error) => logeError('readStream', line, error === null || error === void 0 ? void 0 : error.message)));
                if (!linesCount && tempIndex % bulkSize == 0) {
                    stream.pause();
                    yield Promise.allSettled(pool);
                    pool = [];
                    stream.resume();
                }
                if (linesCount && linesCount - tempIndex < 10) {
                    stream.pause();
                    if (pool.length) {
                        yield Promise.allSettled(pool);
                        pool = [];
                    }
                    yield callback(tempIndex, line);
                    stream.resume();
                }
            })));
            stream.on('error', (error) => {
                emitter.emit('error', error);
                emitter.removeAllListeners();
                rej(error);
            });
            stream.on('end', () => __awaiter(this, void 0, void 0, function* () {
                emitter.emit('end', true);
                emitter.removeAllListeners();
                res();
            }));
        }));
    };
    return { emitter, reader };
}
exports.readStream = readStream;
const countLines = (filePath) => {
    return new Promise((res, rej) => {
        let linesCount = 0;
        (0, fs_1.createReadStream)(filePath)
            .pipe((0, event_stream_1.split)())
            .on('data', (line) => (line === null || line === void 0 ? void 0 : line.trim()) && ++linesCount)
            .on('end', () => res(linesCount))
            .on('error', rej);
    });
};
exports.countLines = countLines;
//# sourceMappingURL=file.js.map